@@ -119,9 +119,19 @@ def get_resource_id(resource, resource_name, name):
             raise osc_exc.CommandError(msg)
         return names[0].get(primary_key)
 
-    except IndexError:
+    except IndexError as e:
         msg = "Unable to locate {0} in {1}".format(name, resource_name)
-        raise osc_exc.CommandError(msg)
+        raise osc_exc.CommandError(msg) from e
+
+
+def add_tags_attr_map(attr_map):
+    tags_attr_map = {
+        'tags': ('tags', list),
+        'any_tags': ('tags-any', list),
+        'not_tags': ('not-tags', list),
+        'not_any_tags': ('not-tags-any', list),
+    }
+    attr_map.update(tags_attr_map)
 
 
 def get_loadbalancer_attrs(client_manager, parsed_args):
@@ -174,8 +184,8 @@ def get_loadbalancer_attrs(client_manager, parsed_args):
             client_manager.load_balancer.flavor_list
         ),
         'availability_zone': ('availability_zone', str),
-
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -227,7 +237,9 @@ def get_listener_attrs(client_manager, parsed_args):
         'allowed_cidrs': ('allowed_cidrs', list),
         'tls_ciphers': ('tls_ciphers', str),
         'tls_versions': ('tls_versions', list),
+        'alpn_protocols': ('alpn_protocols', list),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -275,7 +287,9 @@ def get_pool_attrs(client_manager, parsed_args):
         'disable_tls': ('tls_enabled', lambda x: False),
         'tls_ciphers': ('tls_ciphers', str),
         'tls_versions': ('tls_versions', list),
+        'alpn_protocols': ('alpn_protocols', list),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -317,6 +331,7 @@ def get_member_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -356,6 +371,7 @@ def get_l7policy_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -390,6 +406,7 @@ def get_l7rule_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -428,6 +445,7 @@ def get_health_monitor_attrs(client_manager, parsed_args):
         'http_version': ('http_version', float),
         'domain_name': ('domain_name', str)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -442,6 +460,8 @@ def get_quota_attrs(client_manager, parsed_args):
         'load_balancer': ('load_balancer', int),
         'member': ('member', int),
         'pool': ('pool', int),
+        'l7policy': ('l7policy', int),
+        'l7rule': ('l7rule', int),
         'project': (
             'project_id',
             'project',
@@ -601,8 +621,13 @@ def _format_str_if_need_treat_unset(data):
 
 
 def get_unsets(parsed_args):
-    return {arg: None for arg, value in vars(parsed_args).items() if
-            value is True and arg != 'wait'}
+    unsets = {}
+    for arg, value in vars(parsed_args).items():
+        if value and arg == 'tags':
+            unsets[arg] = value
+        elif value is True and arg not in ('wait', 'all_tag'):
+            unsets[arg] = None
+    return unsets
 
 
 def wait_for_active(status_f, res_id):
@@ -618,7 +643,8 @@ def wait_for_active(status_f, res_id):
             message="The resource did not successfully reach ACTIVE status.")
 
 
-def wait_for_delete(status_f, res_id):
+def wait_for_delete(status_f, res_id,
+                    status_field=constants.PROVISIONING_STATUS):
     class Getter(object):
         @staticmethod
         def get(id):
@@ -628,7 +654,7 @@ def wait_for_delete(status_f, res_id):
         success = utils.wait_for_delete(
             manager=Getter,
             res_id=res_id,
-            status_field=constants.PROVISIONING_STATUS,
+            status_field=status_field,
             sleep_time=3
         )
         if not success:
@@ -638,3 +664,23 @@ def wait_for_delete(status_f, res_id):
     except exceptions.OctaviaClientException as e:
         if e.code != 404:
             raise
+
+
+def set_tags_for_set(resource_get, resource_id, attrs, clear_tags=False):
+    if attrs.get('tags'):
+        resource = resource_get(resource_id)
+        tags = set([] if clear_tags else resource['tags'])
+        tags |= set(attrs['tags'])
+        attrs['tags'] = list(tags)
+    elif clear_tags:
+        attrs['tags'] = []
+
+
+def set_tags_for_unset(resource_get, resource_id, attrs, clear_tags=False):
+    if clear_tags:
+        attrs['tags'] = []
+    elif attrs.get('tags'):
+        resource = resource_get(resource_id)
+        tags = set(resource['tags'])
+        tags -= set(attrs['tags'])
+        attrs['tags'] = list(tags)
